import "package:flutter/material.dart";
import "package:flutter_settings/flutter_settings.dart";

///
class FeedbackConfiguration {
  ///
  const FeedbackConfiguration({
    this.privacyPolicyControls,
    this.termsOfServiceControls,
    this.feedbackControls,
    this.aboutUsControls,
    this.onTapAboutUs,
    this.onTapPrivacyPolicy,
    this.onTapTermsOfService,
    this.onTapFeedback,
    this.privacyPolicyUrl = "https://iconica.app/privacy-en-cookieverklaring/",
    this.termsOfServiceUrl =
        "https://iconica.app/disclaimer-en-algemene-voorwaarden",
    this.selectIssueCategory = true,
    this.reportIssueControls,
    this.onTapReportIssue,
    this.logoUrl = "assets/iconica-logo.png",
    this.showBuildNumber = true,
    this.useAbout = true,
    this.useFeedback = true,
    this.useStoreReview = false,
    this.appStoreId,
    this.customImage,
    this.issueReportedSnackBar,
    this.feedbackPageAlignment,
    this.appBarBuilder,
  })  : assert(
          privacyPolicyControls == null || onTapPrivacyPolicy == null,
          "privacyPolicyControls and onTapPrivacyPolicy cannot both be"
          " non-null at the same time",
        ),
        assert(
          termsOfServiceControls == null || onTapTermsOfService == null,
          "termsOfServiceControls and onTapTermsOfService cannot both be"
          " non-null at the same time",
        ),
        assert(
          feedbackControls == null || onTapFeedback == null,
          "feedbackControls and onTapFeedback cannot both be non-null at the"
          " same time",
        ),
        assert(
          reportIssueControls == null || onTapReportIssue == null,
          "reportIssueControls and onTapReportIssue cannot both be non-null"
          " at the same time",
        );

  ///
  final String privacyPolicyUrl;

  ///
  final String termsOfServiceUrl;

  ///
  final List<Control>? aboutUsControls;

  ///
  final List<Control>? privacyPolicyControls;

  ///
  final List<Control>? termsOfServiceControls;

  ///
  final List<Control>? feedbackControls;

  ///
  final List<Control>? reportIssueControls;

  ///
  final Function()? onTapAboutUs;

  ///
  final Function()? onTapPrivacyPolicy;

  ///
  final Function()? onTapTermsOfService;

  ///
  final Function()? onTapFeedback;

  ///
  final Function()? onTapReportIssue;

  ///
  final bool selectIssueCategory;

  ///
  final String logoUrl;

  ///
  final bool showBuildNumber;

  ///
  final bool useAbout;

  ///
  final bool useFeedback;

  ///
  final bool useStoreReview;

  ///
  final String? appStoreId;

  ///
  final Widget? customImage;

  /// Snackbar that will be shown when a issue has been reported. If null,
  /// no snackbar will be shown.
  final SnackBar? issueReportedSnackBar;

  /// Alignment of the items on the main page of the feedback screen.
  /// If null, the default pf [MainAxisAligment.spaceBetween] will be used.
  final MainAxisAlignment? feedbackPageAlignment;

  /// The builder function for the app bar of the feedback screen.
  /// If null, the default app bar will be used with an elevation of 0.0 to
  /// stop the content and appbar overlapping.
  final FeedbackAppBarBuilder? appBarBuilder;
}

/// Creates an app bar from a [BuildContext] and a title [String].
typedef FeedbackAppBarBuilder = AppBar Function(
  BuildContext context,
  String title,
);
